<?php

namespace Drupal\mapycz\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'mapycz_address' formatter.
 *
 * @FieldFormatter(
 *   id = "mapycz_address",
 *   module = "mapycz",
 *   label = @Translation("Mapy CZ - Address"),
 *   field_types = {
 *     "mapycz"
 *   }
 * )
 */
class MapyCzAddressFormatter extends FormatterBase {

  /**
   * The configuration service.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $mapyczConfig;

  /**
   * Constructs a YourFieldFormatter object.
   *
   * @param string $plugin_id
   *   The plugin ID for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->mapyczConfig = $config_factory->get('mapycz.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $settings = $this->getSettings();
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();
    if (!isset($items[0])) {
      return [];
    }

    $element = [
      '#attached' => [
        'library' => [
          'mapycz/mapycz.frontend',
        ],
        'drupalSettings' => [
          'mapycz' => [
            'api_key' => $this->mapyczConfig->get('api_key'),
          ],
        ],
      ],
    ];

    $map_id = uniqid('mapycz-widget-' . 0);
    $element[0] = [
      '#theme' => 'mapycz_address',
      '#map_id' => $map_id,
      '#marker' => [
        'lat' => $items[0]->lat,
        'lng' => $items[0]->lng,
      ],
      '#theme_wrappers' => [
        'container' => [
          '#attributes' => [
            'id' => $map_id . '-wrapper',
            'class' => 'mapycz-wrapper',
          ],
        ],
      ],
    ];
    return $element;
  }

}
